## $Id: faculty.py 7195 2011-11-25 07:34:07Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Faculty components.
"""

import grok
from zope.component.interfaces import IFactory
from zope.interface import implementedBy
from waeup.sirp.university.interfaces import (
    IFaculty, IFacultyAdd, IDepartment)
from waeup.sirp.university.vocabularies import inst_types

class Faculty(grok.Container):
    """A university faculty.
    """
    grok.implements(IFaculty, IFacultyAdd)

    @property       # Make this method read_only and looking like an attr.
    def local_roles(cls):
        return ['waeup.local.DepartmentManager',
                'waeup.local.ClearanceOfficer',]

    def __init__(self,
                 title=u'Unnamed Faculty',
                 title_prefix=u'faculty',
                 code=u'NA', **kw):
        super(Faculty, self).__init__(**kw)
        self.title = title
        self.title_prefix = title_prefix
        self.code = code

    def addDepartment(self, department):
        """Add a department to the faculty.
        """
        if not IDepartment.providedBy(department):
            raise TypeError('Faculties contain only IDepartment instances')
        self[department.code] = department

    def longtitle(self):
        try:
            result = "%s %s (%s)" % (
                inst_types.getTerm(
                    self.title_prefix).title,
                self.title, self.code)
        except:
            result = "%s (%s)" % (self.title, self.code)
        return result

class FacultyFactory(grok.GlobalUtility):
    """A factory for faculty containers.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.Faculty')
    title = u"Create a new faculty.",
    description = u"This factory instantiates new faculty instances."

    def __call__(self, *args, **kw):
        return Faculty()

    def getInterfaces(self):
        return implementedBy(Faculty)
